/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.crop;

import java.awt.Dimension;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.stock.project.StockProject;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.api.project.Project;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.ExplorerUtils;
import org.openide.filesystems.FileObject;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

/**
 * Visual element to display the crop definition list.
 * Registers itself as multiviewelement on Stockproject.mime_type
 * 
 * @author Timon Veenstra <monezz@gmail.com>
 */
@MultiViewElement.Registration(displayName = "#LBL_Crop_VISUAL",
iconBase = "nl/cloudfarming/client/stock/project/visual/box.png",
mimeType = StockProject.MIME_TYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "CropVisual",
position = 2000)
@NbBundle.Messages({"LBL_Crop_VISUAL=Crop list"})
public class CropListVisualElement implements MultiViewElement, ExplorerManager.Provider, HelpCtx.Provider {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.stock.crop.CropListVisualElement");
    private static final String CROPLIST_ICON = "nl/cloudfarming/client/stock/crop/croplist32.png";
    
    private static final String CROPS_FOLDER = "crops";
    private transient MultiViewElementCallback callback;

    private Lookup lookup;
    private final CropListVisualPanel contentPanel;
    private final ASBodyPanel asBodypanel;

    private final ExplorerManager explorerManager = new ExplorerManager();
    private CropsChildFactory cropsChildFactory;
    private JToolBar emptyDummyToolbar = new JToolBar();
    
    public CropListVisualElement(Lookup lkp) throws IOException {
        
        Project p = lkp.lookup(Project.class);
        assert p != null:"Lookup did not contain a stock project";

        FileObject cropsFolder = getCropsFolder(p.getProjectDirectory(), true);
        CropStorageManager.load(cropsFolder);

        // Create node structure
        cropsChildFactory = new CropsChildFactory(CropStorage.getInstance().getCropDefinitions());
        CropsRootNode cropRootNode = new CropsRootNode(cropsChildFactory);
        explorerManager.setRootContext(cropRootNode);
        
        // Contentpanel
        contentPanel = new CropListVisualPanel(explorerManager);
        asBodypanel = new ASBodyPanel(contentPanel);
               
        asBodypanel.setName(Bundle.LBL_Crop_VISUAL());
        asBodypanel.setIcon(ImageUtilities.loadImageIcon(CROPLIST_ICON, true));
        asBodypanel.setTitle(Bundle.LBL_Crop_VISUAL());
        
        lookup = new ProxyLookup(lkp, ExplorerUtils.createLookup(explorerManager, new ActionMap()));
    }

    @Override
    public ExplorerManager getExplorerManager() {
        return explorerManager;
    }
        
    @Override
    public JComponent getVisualRepresentation() {
        return asBodypanel;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return emptyDummyToolbar; // Toolbar is set on treetableview
    }

    @Override
    public Action[] getActions() {
        return new Action[]{};
    }

    @Override
    public Lookup getLookup() {
        return lookup;
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    private void updateName() {
        if (callback != null) {
            String name = Bundle.LBL_Crop_VISUAL();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }
    
    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.stock.crop.croplist");
    }
        
    /**
     * Returns the underlying crops-folder of the specified projectroot
     * 
     * @param projectRoot Fileobject of the projectroot
     * @param create Wether or not to create the crops-folder.
     * @return Fileobject representation of the crops-folder.
     * @throws IOException 
     */
    private FileObject getCropsFolder(FileObject projectRoot, boolean create) throws IOException {
        FileObject cropsFolder = projectRoot.getFileObject(CROPS_FOLDER);
        if (cropsFolder == null && create) {
            LOGGER.log(Level.FINEST, "Crop folder {0} did not exist, creating now.", CROPS_FOLDER);
            cropsFolder = projectRoot.createFolder(CROPS_FOLDER);
        }
        if (cropsFolder != null && !cropsFolder.isFolder()) {
            LOGGER.log(Level.SEVERE, "crop folder {0} is not a folder, probably a file.", CROPS_FOLDER);
            cropsFolder = null;
        }
        return cropsFolder;
    }
}
