/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.stock.crop;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import nl.cloudfarming.client.crop.model.CropDefinition;
import nl.cloudfarming.client.crop.model.Genus;
import nl.cloudfarming.client.crop.model.Variety;

/**
 * Storage object to store cropdefinitions
 * 
 * @author Timon Veenstra <monezz@gmail.com>
 */
public class CropStorage implements Serializable {

    private static final long serialVersionUID = 1L;
    public static final String TYPE = "nl-cloudfarming-client-stock-crop";
    private List<CropDefinition> cropDefinitions = new ArrayList<>();
    private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
    private static CropStorage instance;
    
    // Property strings used in property change events
    public static final String PROP_CROP_DEFINITION = "cropDefinition";
    public static final String PROP_VARIETY = "variety";
    public static final String PROP_GENUS = "genus";
    
    private CropStorage() {
        List<Variety> varieties = new ArrayList<>();
        varieties.add(new Variety("V1"));
        varieties.add(new Variety("V2"));
        varieties.add(new Variety("V3"));

        List<Genus> gena = new ArrayList<>();
        gena.add(new Genus("A"));
        gena.add(new Genus("B"));
        gena.add(new Genus("C"));

        List<Variety> varieties2 = new ArrayList<>();
        varieties2.add(new Variety("1111111111111111"));
        varieties2.add(new Variety("2222222222222222"));

        List<Genus> gena2 = new ArrayList<>();
        gena2.add(new Genus("Aaaaaaaaaaaaa"));
        gena2.add(new Genus("Bbbbbbbbbbbbb"));

        cropDefinitions.add(new CropDefinition("dummy", varieties, gena));
        cropDefinitions.add(new CropDefinition("dummy2", varieties2, gena2));
    }
    
    /**
     * Get a singleton cropstorage instance
     * @return Singleton instance
     */
    public static CropStorage getInstance(){
        if(instance == null){
            instance = new CropStorage();
        }
        return instance;
    }

    /**
     * Get an unmodifiable list of cropdefinitions in the cropstorage
     * @return The list of cropdefinitions
     */
    public List<CropDefinition> getCropDefinitions() {
        return Collections.unmodifiableList(cropDefinitions);
    }

    /**
     * Remove a cropDefinition from the cropstorage
     * @param cropDefinition Cropdefinition to remove
     */
    public void removeCropDefinition(CropDefinition cropDefinition) {
        cropDefinitions.remove(cropDefinition);
        changeSupport.firePropertyChange(PROP_CROP_DEFINITION, cropDefinition, null);
    }

    /**
     * Remove a genus from the specified cropDefinition
     * @param cropDefinition Cropdefinition to remove a genus from
     * @param genus Genus to remove from the specified cropDefinition
     */
    public void removeGenus(CropDefinition cropDefinition, Genus genus) {
        cropDefinition.removeGenus(genus);
        changeSupport.firePropertyChange(PROP_GENUS, genus, null);
    }
    
    /**
     * Remove a variety from the specified cropDefinition
     * @param cropDefinition Cropdefinition to remove a variety from
     * @param variety Variety to remove from the specified cropDefinition
     */
    public void removeVariety(CropDefinition cropDefinition, Variety variety) {
        cropDefinition.removeVariety(variety);
        changeSupport.firePropertyChange(PROP_VARIETY, variety, null);
    }

    /**
     * Add the specified cropDefinition to the cropStorage and fire propertychange 
     * @param cropDefinition 
     */
    public void addCrop(CropDefinition cropDefinition){
        cropDefinitions.add(cropDefinition);
        changeSupport.firePropertyChange(PROP_CROP_DEFINITION, null, cropDefinition);  
    }
    
    /**
     * Add a genus to the specified cropdefinition
     * @param cropDefinition Cropdefinition to add the genus to
     * @param genus Genus to add to the specified cropdefinition 
     */
    public void addGenus(CropDefinition cropDefinition, Genus genus){
        cropDefinition.addGenus(genus);
        changeSupport.firePropertyChange(PROP_GENUS, null, genus);
    }
    
    /**
     * Add a variety to the specified cropdefinition
     * @param cropDefinition Cropdefinition to add the variety to
     * @param variety Variety to add to the specified cropdefinition 
     */
    public void addVariety(CropDefinition cropDefinition, Variety variety){
        cropDefinition.addVariety(variety);
        changeSupport.firePropertyChange(PROP_VARIETY, null, variety);
    }
    
    /**
     * Add a propertychangelistener to this instance
     * @param property To listen to
     * @param listener The listener to register
     */
    public void addPropertyChangeListener(String property, PropertyChangeListener listener){
        changeSupport.addPropertyChangeListener(property, listener);
    }
    
    /**
     * Remove a propertychangelistener from this instance
     * @param property The property the registered listener is listening to
     * @param listener The registered listener to remove
     */
    public void removePropertyChangeListener(String property, PropertyChangeListener listener){
        changeSupport.removePropertyChangeListener(property, listener);
    }
}
